//+------------------------------------------------------------------+
//|                                         FX Steady Growth .mq4    |
//|                                                                  |
//| I have quite a good EA, but I think it would work even better    |
//| when I add a filter to it.                                       |
//|                                                                  |
//| Firstly, I will describe, how it works now:                      |
//| It is based on MA                                                |
//|  when a candle crosses MA, it opens a trade by the beginning of  |
//|  the next candle (in the same direction as it crosse the candle).|
//|  The closing condition is as follows:                            |
//|  when the MA crosses candle in the opposite direction,           |
//|  or when it hits SL/TSL.
//|                                                                  |
//| Now I would like to suggest a possible change to it:             |
//| There would be one more condition.                               |
//|  We open a trade only when the candle crosses MA                 |
//|  and at the same time both the candle and the MA                 |
//|  must be rising(and vise versa).                                 |
//|  If the candle is going up, but the MA is still on decrease,     |
//|  we dont open a trade (and vise versa).                          |
//|  We postpone a trade and execute it by the time the MA turns to  |
//|  the same direction as the candle.                               |
//|  We close the current trade at the same time we open a new one   |
//| (bacically, we have one opened position all the time).           |
//|                                                                  |
//| 9/28/2011 : Modified by Robert Hill aka MrPip                    |
//|             comments added to code changes                       |
//+------------------------------------------------------------------+

#define MagicNumber  20050555

// Add defines for signals by Robert
#define LONG 1
#define SHORT -1
#define FLAT 0

extern bool   MoneyManagement    = false;     
extern double LotsPercent        = 2;       
extern double ManualLotSize      = 0.1;      
extern double StopLoss           = 55;
extern double TakeProfit         = 800;
extern double MovingPeriod       = 21;
extern double MovingShift        = 12;
extern int    TrailingStop       = 65;
extern int    Slippage           = 3;

double myPoint;  // added by Robert : this handles 5 digit pricing correctly

//+------------------------------------------------------------------+
int init()
  {
//----
//   if(BrokerIs5Digit())
//   {
//      StopLoss*=10;
//      TakeProfit*=10;
//      Slippage*=10;
//      TrailingStop*=10;
//   }
   myPoint = SetPoint(); // Roberts better way to handle 5 digit brokers, replace Point with myPoint
   
//----
   return(0);
  }

double SetPoint()
{
   double mPoint;
   
   if (Digits < 4)
      mPoint = 0.01;
   else
      mPoint = 0.0001;
   
   return(mPoint);
}

//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
//      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MagicNumber)
      if(OrderSymbol()==symbol && OrderMagicNumber()==MagicNumber)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
  double MinLot=NormalizeDouble(MarketInfo(Symbol(),MODE_MINLOT),2);
  double MaxLot=NormalizeDouble(MarketInfo(Symbol(),MODE_MAXLOT),2);
  double LotStep=NormalizeDouble(MarketInfo(Symbol(),MODE_LOTSTEP),2);
  double FullMargin=MarketInfo(Symbol(),MODE_MARGINREQUIRED);
  double LotSize,Lot;
  int LotDigit;
  string MM="";
  
  //////////////////////////////  
  if(MoneyManagement==true)
  {
  LotSize=(AccountBalance()/100000)*LotsPercent;
  }
  ////////////////////////////////////////////////// 
  if(MoneyManagement==false) LotSize=ManualLotSize;
  /////////////////////////////////////////////
  if(LotSize<MinLot) LotSize=MinLot;
  if(LotSize>MaxLot) LotSize=MaxLot;
  /////////////////////////////////////////////
  if(LotStep==1) LotDigit=0;
  if(LotStep==0.1) LotDigit=1;
  if(LotStep==0.01) LotDigit=2;
  
  //////////////////////////////////////////
  Lot= MathFloor(LotSize/LotStep)*LotStep;
   return(Lot);
  }
  
// Robert put code for entry and exit in one place
// This makes it much easier to change signal requirements
// for opening and closing reades
int GetSignal()
{
   double maCur, maPrev;
//---- get Moving Average 
   maCur=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
   maPrev=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,1);
//---- sell conditions

   if(Open[1]>maCur && Close[1]<maCur) // This also meets condition of candle moving down 
   {
      if (maCur < maPrev) return(SHORT); // added check for falling ma
   }
//---- buy conditions
   if(Open[1]<maCur && Close[1]>maCur) // this also meets the condition of candle moving up  
   {
      if (maCur > maPrev) return(LONG); // added chack for rising ma
   }
   return(FLAT);

}

//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
{
   int    signal, res;
   
//---- go trading only for first tiks of new bar
   static datetime Time0; if (Time0 == Time[0]) return(0); Time0 = Time[0];

   signal = GetSignal(); // This makes it easy to change entry signal requirements
   if (signal == SHORT) OpenSell("");
   if (signal == LONG) OpenBuy("");
}
//+------------------------------------------------------------------+
//| Trailing Stop function                                           |
//+------------------------------------------------------------------+
void Trailing_Stop()
{
   double pt, SL, OSL;
   
   for(int cnt = OrdersTotal() - 1; cnt >= 0; cnt--)
   {
      OrderSelect(cnt, SELECT_BY_POS, MODE_TRADES);
      //----
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == MagicNumber)
      {       
			pt = TrailingStop * myPoint; // Robert simplified code
         OSL = OrderStopLoss();
         OSL = NormalizeDouble( OSL, Digits); // fixes MT4 bug for broker returning too many digits for OSL
	  	   if(OrderType() == OP_BUY)
	  	   {
	  		   if(TrailingStop > 0) 
	  		   {
			      if(Bid - OrderOpenPrice() >= pt)
			      {
			         SL = Bid - pt;
                  SL = NormalizeDouble( SL, Digits);
	               if(OSL < SL) 
                  {					   
				         OrderModify(OrderTicket(), OrderOpenPrice(), SL, 0, 0, Yellow);
				         return(0);
	  		         }
	  		      }
	  		   }
	  		}
	  	   if(OrderType() == OP_SELL) 
	  	   {
	  	      if(TrailingStop > 0) 
	  	      {
		         if(OrderOpenPrice() - Ask >= pt)
		         {
			         SL = Ask + pt;
                  SL = NormalizeDouble( SL, Digits);
		            if(OSL > SL)
		            {
		               OrderModify(OrderTicket(), OrderOpenPrice(), SL, 0, 0, Purple);
		               return(0);
		            }
   	         }
            } 
         }
      }
   }
}

//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
{
   int signal;
//---- go trading only for first tiks of new bar
   static datetime Time0;    
   
   if(Time0 == Time[0]) return(0); 
   
   Time0 = Time[0];
   
   bool res = false; int retry = 0;
//----
   for(int i=OrdersTotal()-1; i>=0; i--)
   {      
      if(OrderSelect(i, SELECT_BY_POS))
      {
        if (OrderMagicNumber()  == MagicNumber &&  OrderSymbol() == Symbol() )
        {
          signal = GetSignal(); // This makes it easy to change exit signal requirements
          
        //---- check order type 
         if(OrderType()==OP_BUY)
         {
            if(signal == SHORT) 
            {
               while(!IsTradeAllowed()) Sleep(100);
               while(IsTradeContextBusy()) Sleep(100);
         
               while(retry < 5 && !res)
               {
                  RefreshRates();
                  res = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,White);
                  
                  Sleep(500);
                  retry++;
               }
               
               if(!res) Print(Symbol()," Unable to close buy trade! Error: ",GetLastError());
            }
         }
         if(OrderType()==OP_SELL)
         {
            if(signal == LONG) 
            {
               while(!IsTradeAllowed()) Sleep(100);
               while(IsTradeContextBusy()) Sleep(100);
               
               while(retry < 5 && !res)
               {
                  RefreshRates();
               
                  res = OrderClose(OrderTicket(),OrderLots(),OrderClosePrice(),Slippage,White);
                  
                  Sleep(500);
                  retry++;
               }
               
               if(!res) Print(Symbol()," Unable to close sell trade! Error: ",GetLastError());
            }
         }
       }
      }
   }
//----
}

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
{
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
   
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) 
   {
      CheckForOpen();
   }
   else
   {
      Trailing_Stop() ;
      CheckForClose() ;
   }
   Comment("Forex Steady Growth status OK");
}
//+------------------------------------------------------------------+---+


// Robert modified to place sl and tp inrelation to OrderOpenPrice value instead of expected entry
// Also added additional code to prevent errors from ordermodify calls
void OpenBuy(string comment)
{
   int ticket = -1;
   int retry = 0;
   
   double SL,TP;
   
   
   while(!IsTradeAllowed()) Sleep(100);
   while(IsTradeContextBusy()) Sleep(100);
   
   while(ticket <= 0 && retry < 5)
   {
      RefreshRates();
      ticket = OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,Slippage,0,0,comment,MagicNumber,0,Blue);
      retry++;
      Sleep(200);
   }
   
   if(ticket > 0)
   {
      if(StopLoss > 0 || TakeProfit > 0)
      {
         OrderSelect(ticket,SELECT_BY_TICKET);
         
         while(!IsTradeAllowed()) Sleep(50);
         while(IsTradeContextBusy()) Sleep(50);
         
         if(StopLoss > 0) SL = OrderOpenPrice() - StopLoss*myPoint;
         else SL = 0;
         if(TakeProfit > 0) TP = OrderOpenPrice() + TakeProfit*myPoint;
         else TP = 0;
         
 // Normalize stoploss / takeprofit to the proper # of digits.
         if (Digits > 0) 
         {
           SL = NormalizeDouble( SL, Digits);
           TP = NormalizeDouble( TP, Digits); 
         }
         
         if(!OrderModify(OrderTicket(),OrderOpenPrice(),SL,TP,0,CLR_NONE)) 
         {
            Print(Symbol()," Unable to set SL/TP! Error: ",GetLastError());
            Print("Ask: ",Ask, " SL: ",SL, " TP: ",TP);
         }
      }
   }
   else Print(Symbol()," Unable to open buy trade! Error: ",GetLastError());
}

// Robert modified to place sl and tp inrelation to OrderOpenPrice value instead of expected entry
// Also added additional code to prevent errors from ordermodify calls
void OpenSell(string comment)
{
   int ticket = -1;
   int retry = 0;
   
   double SL,TP;
   
   
   while(!IsTradeAllowed()) Sleep(100);
   while(IsTradeContextBusy()) Sleep(100);
   
   while(ticket <= 0 && retry < 5)
   {
      RefreshRates();
      ticket = OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,Slippage,0,0,comment,MagicNumber,0,Red);
      retry++;
      Sleep(200);
   }
   
   if(ticket > 0)
   {
      if(StopLoss > 0 || TakeProfit > 0)
      {
         OrderSelect(ticket,SELECT_BY_TICKET);
         
         while(!IsTradeAllowed()) Sleep(50);
         while(IsTradeContextBusy()) Sleep(50);
         
         if(StopLoss > 0) SL = OrderOpenPrice() + StopLoss*myPoint;
         else SL = 0;
         if(TakeProfit > 0) TP = OrderOpenPrice() - TakeProfit*myPoint;
         else TP = 0;
         
 // Normalize stoploss / takeprofit to the proper # of digits.
         if (Digits > 0) 
         {
           SL = NormalizeDouble( SL, Digits);
           TP = NormalizeDouble( TP, Digits); 
         }
         
         if(!OrderModify(OrderTicket(),OrderOpenPrice(),SL,TP,0,CLR_NONE)) 
         {
            Print(Symbol()," Unable to set SL/TP! Error: ",GetLastError());
            Print("Bid: ",Bid, " SL: ",SL, " TP: ",TP);
         }
      }
   }
   else Print(Symbol()," Unable to open sell trade! Error: ",GetLastError());
}

// The following 2 functions are no longer needed
// So Robert commented them out
/*
int SymbolsList(string & Symbols[])
{
   int Offset, SymbolsNumber;
   
   string SymbolsFileName = "symbols.sel";
 
   int hFile = FileOpenHistory(SymbolsFileName, FILE_BIN|FILE_READ);
   if(hFile < 0) return(-1);
 
   SymbolsNumber = (FileSize(hFile)-4)/128; 
   Offset = 116;
   
   ArrayResize(Symbols, SymbolsNumber);
 
   FileSeek(hFile, 4, SEEK_SET);
   
   for(int i=0;i<SymbolsNumber;i++)
   {
      Symbols[i] = FileReadString(hFile, 12);
      FileSeek(hFile, Offset, SEEK_CUR);
   }
   
   FileClose(hFile);
 
   return(SymbolsNumber);
}

bool BrokerIs5Digit()
{
   string symbols[];
   string eurusd = "";
   
   if(SymbolsList(symbols) > 0)
   {
      for(int i=0;i<ArraySize(symbols);i++)
      {
         if(StringFind(symbols[i],"EURUSD",0) != -1) 
         {
            eurusd = symbols[i];
            break;
         }
      }
      
      if(eurusd != "") if(MarketInfo(eurusd,MODE_DIGITS)==5) return(true);
   }
   
   return(false);
}
*/

